"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var RAFHelper_1 = require("../../../utils/RAFHelper");
var ItemAnimator_1 = require("../../../core/ItemAnimator");
/**
 * This animator prevents views from shifting up/down.
 * This is stateful and thus, will work once per mount. Make sure you pass a new instance for every
 * new RLV mount.
 */
var LayoutStabilizerAnimator = /** @class */ (function (_super) {
    __extends(LayoutStabilizerAnimator, _super);
    function LayoutStabilizerAnimator(numberOfFramesToBatch) {
        if (numberOfFramesToBatch === void 0) { numberOfFramesToBatch = 3; }
        var _this = _super.call(this) || this;
        _this._isQueueComplete = false;
        _this._stabilizationInProgress = false;
        _this._stabilizationComplete = false;
        _this._lastProcessingTS = 0;
        _this._indexBounds = { min: 0, max: 0 };
        _this._viewCache = [];
        _this._numberOfFramesToBatch = numberOfFramesToBatch;
        return _this;
    }
    LayoutStabilizerAnimator.prototype.shouldPersistStyleOverrides = function () {
        return !this._isStabilizationComplete();
    };
    LayoutStabilizerAnimator.prototype.animateWillMount = function (atX, atY, itemIndex) {
        if (this.isRecreateFlow && !this._isStabilizationComplete()) {
            this._completeStabilization();
            this._isQueueComplete = true;
        }
        else if (!this._isQueueComplete) {
            this._indexBounds.min = Math.min(itemIndex, this._indexBounds.min);
            this._indexBounds.max = Math.max(itemIndex, this._indexBounds.max);
            return { opacity: 0 };
        }
        return undefined;
    };
    LayoutStabilizerAnimator.prototype.animateDidMount = function (atX, atY, itemRef, itemIndex) {
        if (!this._isQueueComplete && itemIndex >= this._indexBounds.min && itemIndex <= this._indexBounds.max) {
            if (this._viewCache) {
                this._viewCache.push(itemRef);
                if (itemIndex === this._indexBounds.max) {
                    this._isQueueComplete = true;
                }
            }
            this._checkForMovementAndStabilize(itemIndex);
        }
    };
    LayoutStabilizerAnimator.prototype.animateWillUpdate = function (fromX, fromY, toX, toY, itemRef, itemIndex) {
        this._checkForMovementAndStabilize(itemIndex);
    };
    LayoutStabilizerAnimator.prototype.animateShift = function (fromX, fromY, toX, toY, itemRef, itemIndex) {
        this._checkForMovementAndStabilize(itemIndex);
        return false;
    };
    LayoutStabilizerAnimator.prototype.animateWillUnmount = function (atX, atY, itemRef, itemIndex) {
        itemRef._isUnmountedForRecyclerListView = true;
    };
    LayoutStabilizerAnimator.prototype._checkForMovementAndStabilize = function (itemIndex) {
        var _this = this;
        if (itemIndex >= this._indexBounds.min && itemIndex <= this._indexBounds.max) {
            var currentTS_1 = Date.now();
            this._lastProcessingTS = Date.now();
            if (!this._hasStabilizationStartedOrComplete()) {
                this._startStabilization();
                RAFHelper_1.RAFHelper.executeOnRAF(function (token) {
                    if (_this._lastProcessingTS === currentTS_1) {
                        token.terminate();
                        _this._completeStabilization();
                        _this._unHideViews();
                    }
                    else {
                        currentTS_1 = _this._lastProcessingTS;
                    }
                }, this._numberOfFramesToBatch);
            }
        }
    };
    LayoutStabilizerAnimator.prototype._startStabilization = function () {
        this._stabilizationInProgress = true;
    };
    LayoutStabilizerAnimator.prototype._completeStabilization = function () {
        this._stabilizationInProgress = false;
        this._stabilizationComplete = true;
    };
    LayoutStabilizerAnimator.prototype._hasStabilizationStartedOrComplete = function () {
        return this._stabilizationInProgress || this._stabilizationComplete;
    };
    LayoutStabilizerAnimator.prototype._isStabilizationComplete = function () {
        return this._stabilizationComplete;
    };
    LayoutStabilizerAnimator.prototype._unHideViews = function () {
        if (this._viewCache) {
            this._viewCache.forEach(function (arrItem) {
                var view = arrItem;
                if (!view._isUnmountedForRecyclerListView) {
                    if (view.setNativeProps) {
                        view.setNativeProps({ style: { opacity: 1 } });
                    }
                    else if (view.style) {
                        view.style.opacity = 1;
                    }
                }
            });
            this._viewCache = undefined;
        }
    };
    return LayoutStabilizerAnimator;
}(ItemAnimator_1.BaseItemAnimator));
exports.LayoutStabilizerAnimator = LayoutStabilizerAnimator;
//# sourceMappingURL=LayoutStabilizerAnimator.js.map